////////////////////////////////////////////////////////////////////////////////////////////////////////
//      Made by Shaposhnikov Maxim, town Stavropol, Russia       ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
#if !defined(MAX_INET2_HPP)
#define MAX_INET2_HPP
#include <vector>
#include <cstring>
#include <boost/thread.hpp>
#include <boost/function.hpp>
#include <boost/bind.hpp>
const unsigned short int majorv=2;
const unsigned short int minorv=3;
#if defined (WIN32) ||defined (WIN64)
#include <winsock2.h>
#include <windows.h>


namespace maxis11
{

class MInet_Log // ,  
{
private:
    std::vector<std::string>log_str;
public:
    void add_log(std::string);
    std::vector<std::string> print_log ();
};

class MGlobal_Inet //  
{
private:
    bool WSA_started;
    WSADATA ws;
    MInet_Log log;
public:
    void WSA_start();
    void WSA_exit();
    std::vector<std::string> print_log();
    WSADATA print_WSA();
}Glob;

class MTCP_client // 
{
private:
    u_short port;
    std::string servip;
    MInet_Log log;
    bool thread_start;
    SOCKET sock;
    sockaddr_in serv_addr;
    boost::thread list_thread;
    void* datai;
    void tech_listen(boost::function<bool (SOCKET,void*)>);
    void reload_sock();
    void start_param();
public:
    MTCP_client();
    MTCP_client(u_short,std::string);
    void set_address (u_short,std::string);
    void connect_serv(boost::function<bool (SOCKET,void*)>,void*);
    std::vector<std::string> print_log();
    void join_in_main();
    ~MTCP_client();
};

class MTCP_server // 
{
private:
    MInet_Log log;
    u_short port;
    std::string ip;
    SOCKET sock;
    sockaddr_in addr;
    boost::thread main_thread;
    void* datai;
    void reload_sock();
    void tech_listen (boost::function<bool (SOCKET,sockaddr_in,void*)>);
    void tech_listen_client(boost::function<bool (SOCKET,sockaddr_in,void*)>,sockaddr_in, SOCKET);
public:
    MTCP_server();
    MTCP_server(u_short,std::string);
    void set_address (u_short,std::string);
    void bindserv();
    void listenserv(boost::function<bool (SOCKET,sockaddr_in,void*)>,int,void*);
    void stoplisten ();
    void interrupt();
    std::vector<std::string> print_log();
    ~MTCP_server();
};

class MUDP_client // 
{
private:
    bool thread_start;
    std::vector<u_short> port;
    std::vector<std::string> servip;
    MInet_Log log;
    SOCKET sock;
    std::vector<sockaddr_in> serv_addrs;
    boost::thread list_thread;
    void* datai;
    void reload_sock();
    void tech_listen(boost::function<bool (SOCKET,std::vector<sockaddr_in>,void*)>);
public:
    MUDP_client();
    MUDP_client(std::vector<u_short>,std::vector<std::string>);
    void set_addreses (std::vector<u_short>,std::vector<std::string>);
    void add_address (u_short,std::string);
    void interact_servs(boost::function<bool (SOCKET,std::vector<sockaddr_in>,void*)>,void*);
    std::vector<std::string> print_log();
    void join_in_main();
    ~MUDP_client();
};

class MUDP_server // 
{
private:
    bool listening;
    MInet_Log log;
    u_short port;
    std::string ip;
    SOCKET sock;
    sockaddr_in addr;
    boost::thread main_thread;
    void* datai;
    void reload_sock();
    void tech_listen (boost::function<bool (SOCKET,void*)>);
public:
    MUDP_server();
    MUDP_server(u_short,std::string);
    void set_address (u_short,std::string);
    void bindserv();
    void interact_clients (boost::function<bool (SOCKET,void*)>,void*);
    void join_in_main();
    void interrupt();
    std::vector<std::string> print_log();
    ~MUDP_server();
};

void MInet_Log::add_log(std::string s)
{
    log_str.push_back(s);
}

std::vector<std::string> MInet_Log::print_log()
{
    return log_str;
}

void MGlobal_Inet::WSA_start()
{
    if (WSA_started==0) {
        if (WSAStartup(MAKEWORD( 2, 2 ), &ws))
        {
            std::stringstream sstr;
            sstr<<"WSA_start "<<WSAGetLastError()<<std::endl;
           log.add_log(sstr.str());
        }
        else
        {
            WSA_started=1;
            log.add_log("WSA_start 0\n");
        }
    }
}

void MGlobal_Inet::WSA_exit()
{
    if (WSA_started==1)
    {
        WSACleanup();
        WSA_started=0;
        log.add_log("WSA_exit 0\n");
    }
}

WSADATA MGlobal_Inet::print_WSA()
{
    return ws;
}

std::vector<std::string> MGlobal_Inet::print_log()
{
    return log.print_log();
}

MTCP_client::MTCP_client()
{
    start_param();
    Glob.WSA_start();
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MTCP_client "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("MTCP_client 0\n");
    }
}

MTCP_client::MTCP_client(u_short porti, std::string ip)
{
    start_param();
    Glob.WSA_start();
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MTCP_client "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        port=porti;
        servip=ip;
        log.add_log("MTCP_client 0\n");
    }
}

void MTCP_client::set_address(u_short porti,std::string ip)
{
   port=porti;
   servip=ip;
}

void MTCP_client::connect_serv(boost::function<bool (SOCKET,void*)> f, void* data)
{
    datai=data;
    serv_addr.sin_family=AF_INET;
    serv_addr.sin_port=htons(port);
    if (inet_addr(servip.c_str())!=INADDR_NONE)
    {
        serv_addr.sin_addr.s_addr=inet_addr(servip.c_str());
        if (connect(sock,(sockaddr*)&serv_addr,sizeof(serv_addr)))
        {
            std::stringstream sstr;
            sstr<<"connect_serv_connection "<<WSAGetLastError()<<std::endl;
            log.add_log(sstr.str());
            reload_sock();
        }
        else
        {
            log.add_log("connect_serv 0\n");
            list_thread=boost::thread ( boost::bind( &MTCP_client::tech_listen, this, f ) );
            thread_start=1;
        }
    }
    else
    {
        std::stringstream sstr;
        sstr<<"connect_serv_ip "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
}

void MTCP_client::tech_listen(boost::function<bool (SOCKET,void*)> f)
{
    bool exitwh=0;
    while (exitwh==0)
    {
        exitwh=f(sock,datai);
    }
    log.add_log("tech_listen 0");
    reload_sock();
    thread_start=0;
}

void MTCP_client::reload_sock()
{
    closesocket(sock);
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"reload_sock "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("reload_sock 0\n");
    }
}

std::vector<std::string> MTCP_client::print_log()
{
    return log.print_log();
}

void MTCP_client::start_param()
{
    port=0;
    thread_start=0;
}

void MTCP_client::join_in_main()
{
    if (thread_start==1)
        list_thread.join();
}

MTCP_client::~MTCP_client()
{
    if (thread_start==1)
        list_thread.join();
    closesocket(sock);
    Glob.WSA_exit();
}

MUDP_client::MUDP_client()
{
    thread_start=0;
    Glob.WSA_start();
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MUDP_client "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("MUDP_client 0\n");
    }
}

MUDP_client::MUDP_client(std::vector<u_short>porti,std::vector<std::string>ip)
{
    thread_start=0;
    Glob.WSA_start();
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MUDP_client "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        port=porti;
        servip=ip;
        log.add_log("MUDP_client 0\n");
    }
}

void MUDP_client::set_addreses(std::vector<u_short>porti, std::vector<std::string>ip)
{
    port=porti;
    servip=ip;
}

void MUDP_client::add_address(u_short porti,std::string ip)
{
    port.push_back(porti);
    servip.push_back(ip);
}

void MUDP_client::reload_sock()
{
    closesocket(sock);
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"reload_sock "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("reload_sock 0\n");
    }
}

std::vector<std::string> MUDP_client::print_log()
{
    return log.print_log();
}

void MUDP_client::interact_servs(boost::function<bool (SOCKET,std::vector<sockaddr_in>,void*)> f, void* data)
{
    bool err=0;
    datai=data;
    for (u_int i=0; i<servip.size(); i++)
    {
        sockaddr_in serv_addr;
        serv_addr.sin_family=AF_INET;
        serv_addr.sin_port=htons(port[i]);
        if (inet_addr(servip[i].c_str())!=INADDR_NONE)
        {
            serv_addr.sin_addr.s_addr=inet_addr(servip[i].c_str());
            serv_addrs.push_back(serv_addr);
        }
        else
        {
            err=1;
            break;
        }
    }
    if (err==0)
    {
        log.add_log("interact_servs 0\n");
        list_thread=boost::thread ( boost::bind( &MUDP_client::tech_listen, this, f ) );
        thread_start=1;
    }
    else
    {
        std::stringstream sstr;
        sstr<<"interact_servs_ip "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
}

void MUDP_client::tech_listen(boost::function<bool (SOCKET,std::vector<sockaddr_in>,void*)> f)
{
    bool exitwh=0;
    while (exitwh==0)
    {
        exitwh=f(sock,serv_addrs,datai);
    }
    log.add_log("tech_listen 0");
    reload_sock();
    thread_start=0;
}

void MUDP_client::join_in_main()
{
    if (thread_start==1)
        list_thread.join();
}

MUDP_client::~MUDP_client()
{
    if (thread_start==1)
        list_thread.join();
    closesocket(sock);
    Glob.WSA_exit();
}

MTCP_server::MTCP_server()
{
    Glob.WSA_start();
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MTCP_server "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("MTCP_server 0\n");
    }
}

MTCP_server::MTCP_server(u_short porti, std::string ipi)
{
    Glob.WSA_start();
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MTCP_server "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        ip=ipi;
        port=porti;
        addr.sin_family=AF_INET;
        addr.sin_port=htons(port);
        addr.sin_addr.s_addr=inet_addr(ip.c_str());
        log.add_log("MTCP_server 0\n");
    }
}

void MTCP_server::set_address(u_short porti,std::string ipi)
{
    ip=ipi;
    port=porti;
    addr.sin_family=AF_INET;
    addr.sin_port=htons(port);
    addr.sin_addr.s_addr=inet_addr(ip.c_str());
}

void MTCP_server::reload_sock()
{
    closesocket(sock);
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"reload_sock "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
        log.add_log("reload_sock 0\n");
}

void MTCP_server::bindserv()
{
    if (bind(sock,(sockaddr *) &addr,sizeof(addr)))
    {
        std::stringstream sstr;
        sstr<<"bindserv "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
    else
        log.add_log("bindserv 0\n");
}

void MTCP_server::listenserv (boost::function<bool (SOCKET,sockaddr_in,void*)> f,int qri, void* data)
{
    datai=data;
    if (listen(sock, qri))
    {
        std::stringstream sstr;
        sstr<<"listenserv "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
    else
    {
        main_thread=boost::thread ( boost::bind( &MTCP_server::tech_listen, this, f) );
        log.add_log("listenserv 0\n");
    }
}

void MTCP_server::tech_listen(boost::function<bool (SOCKET,sockaddr_in,void*)> f)
{
    SOCKET client_sock;
    sockaddr_in client_addr;
    int client_addr_size=sizeof(client_addr);
    while((client_sock=accept(sock, (sockaddr *)&client_addr, &client_addr_size))!=SOCKET_ERROR)
    {
        boost::thread cl_thread=boost::thread ( boost::bind( &MTCP_server::tech_listen_client, this, f,client_addr,client_sock));
    }
    log.add_log("tech_listen 0 \n");
    reload_sock();
}

void MTCP_server::tech_listen_client(boost::function<bool (SOCKET,sockaddr_in,void*)> f,sockaddr_in client_addr, SOCKET client_sock)
{
    bool exitwh=0;
    while (exitwh==0)
    {
        exitwh=f(client_sock,client_addr,datai);
    }
    closesocket(client_sock);
    log.add_log("tech_listen_client 0");
}

std::vector<std::string> MTCP_server::print_log()
{
    return log.print_log();
}

void MTCP_server::interrupt()
{
    main_thread.interrupt();
}

void MTCP_server::stoplisten()
{
    main_thread.interrupt();
    reload_sock();
}

MTCP_server::~MTCP_server ()
{
    stoplisten();
    closesocket(sock);
    Glob.WSA_exit();
}

MUDP_server::MUDP_server()
{
    Glob.WSA_start();
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MUDP_server "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("MUDP_server 0\n");
    }
}

MUDP_server::MUDP_server(u_short porti, std::string ipi)
{
    Glob.WSA_start();
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MUDP_server "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        ip=ipi;
        port=porti;
        addr.sin_family=AF_INET;
        addr.sin_port=htons(port);
        addr.sin_addr.s_addr=inet_addr(ip.c_str());
        log.add_log("MUDP_server 0\n");
    }
}

void MUDP_server::set_address(u_short porti,std::string ipi)
{
    ip=ipi;
    port=porti;
    addr.sin_family=AF_INET;
    addr.sin_port=htons(port);
    addr.sin_addr.s_addr=inet_addr(ip.c_str());
}

void MUDP_server::reload_sock()
{
    closesocket(sock);
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"reload_sock "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
        log.add_log("reload_sock 0\n");
}

void MUDP_server::bindserv()
{
    if (bind(sock,(sockaddr *) &addr,sizeof(addr)))
    {
        std::stringstream sstr;
        sstr<<"bindserv "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
    else
        log.add_log("bindserv 0\n");
}

void MUDP_server::interact_clients(boost::function<bool (SOCKET,void*)> f, void* data)
{
    datai=data;
    main_thread=boost::thread ( boost::bind( &MUDP_server::tech_listen, this, f ) );
}

void MUDP_server::tech_listen(boost::function<bool (SOCKET,void*)> f)
{
    bool exitwh=0;
    while (exitwh==0)
    {
        exitwh=f(sock,datai);
    }
    reload_sock();
    log.add_log("tech_listen 0");
}

void MUDP_server::join_in_main()
{
    main_thread.join();
}

void MUDP_server::interrupt()
{
    main_thread.interrupt();
}

MUDP_server::~MUDP_server()
{
    interrupt();
    closesocket(sock);
    Glob.WSA_exit();
}
};
#elif defined(__unix__)
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

namespace maxis11
{

class MInet_Log // ,  
{
private:
    std::vector<std::string>log_str;
public:
    void add_log(std::string);
    std::vector<std::string> print_log ();
};

class MTCP_client // 
{
private:
    u_short port;
    std::string servip;
    MInet_Log log;
    bool thread_start;
    SOCKET sock;
    sockaddr_in serv_addr;
    boost::thread list_thread;
    void* datai;
    void tech_listen(boost::function<bool (SOCKET,void*)>);
    void reload_sock();
    void start_param();
public:
    MTCP_client();
    MTCP_client(u_short,std::string);
    void set_address (u_short,std::string);
    void connect_serv(boost::function<bool (SOCKET,void*)>,void*);
    std::vector<std::string> print_log();
    void join_in_main();
    ~MTCP_client();
};

class MTCP_server // 
{
private:
    MInet_Log log;
    u_short port;
    std::string ip;
    SOCKET sock;
    sockaddr_in addr;
    boost::thread main_thread;
    void* datai;
    void reload_sock();
    void tech_listen (boost::function<bool (SOCKET,sockaddr_in,void*)>);
    void tech_listen_client(boost::function<bool (SOCKET,sockaddr_in,void*)>,sockaddr_in, SOCKET);
public:
    MTCP_server();
    MTCP_server(u_short,std::string);
    void set_address (u_short,std::string);
    void bindserv();
    void listenserv(boost::function<bool (SOCKET,sockaddr_in,void*)>,int,void*);
    void stoplisten ();
    void interrupt();
    std::vector<std::string> print_log();
    ~MTCP_server();
};

class MUDP_client // 
{
private:
    bool thread_start;
    std::vector<u_short> port;
    std::vector<std::string> servip;
    MInet_Log log;
    SOCKET sock;
    std::vector<sockaddr_in> serv_addrs;
    boost::thread list_thread;
    void* datai;
    void reload_sock();
    void tech_listen(boost::function<bool (SOCKET,std::vector<sockaddr_in>,void*)>);
public:
    MUDP_client();
    MUDP_client(std::vector<u_short>,std::vector<std::string>);
    void set_addreses (std::vector<u_short>,std::vector<std::string>);
    void add_address (u_short,std::string);
    void interact_servs(boost::function<bool (SOCKET,std::vector<sockaddr_in>,void*)>,void*);
    std::vector<std::string> print_log();
    void join_in_main();
    ~MUDP_client();
};

class MUDP_server // 
{
private:
    bool listening;
    MInet_Log log;
    u_short port;
    std::string ip;
    SOCKET sock;
    sockaddr_in addr;
    boost::thread main_thread;
    void* datai;
    void reload_sock();
    void tech_listen (boost::function<bool (SOCKET,void*)>);
public:
    MUDP_server();
    MUDP_server(u_short,std::string);
    void set_address (u_short,std::string);
    void bindserv();
    void interact_clients (boost::function<bool (SOCKET,void*)>,void*);
    void join_in_main();
    void interrupt();
    std::vector<std::string> print_log();
    ~MUDP_server();
};

void MInet_Log::add_log(std::string s)
{
    log_str.push_back(s);
}

MTCP_client::MTCP_client()
{
    start_param();
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MTCP_client "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("MTCP_client 0\n");
    }
}

MTCP_client::MTCP_client(u_short porti, std::string ip)
{
    start_param();
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MTCP_client "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        port=porti;
        servip=ip;
        log.add_log("MTCP_client 0\n");
    }
}

void MTCP_client::set_address(u_short porti,std::string ip)
{
   port=porti;
   servip=ip;
}

void MTCP_client::connect_serv(boost::function<bool (SOCKET,void*)> f, void* data)
{
    datai=data;
    serv_addr.sin_family=AF_INET;
    serv_addr.sin_port=htons(port);
    if (inet_addr(servip.c_str())!=INADDR_NONE)
    {
        serv_addr.sin_addr.s_addr=inet_addr(servip.c_str());
        if (connect(sock,(sockaddr*)&serv_addr,sizeof(serv_addr)))
        {
            std::stringstream sstr;
            sstr<<"connect_serv_connection "<<WSAGetLastError()<<std::endl;
            log.add_log(sstr.str());
            reload_sock();
        }
        else
        {
            log.add_log("connect_serv 0\n");
            list_thread=boost::thread ( boost::bind( &MTCP_client::tech_listen, this, f ) );
            thread_start=1;
        }
    }
    else
    {
        std::stringstream sstr;
        sstr<<"connect_serv_ip "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
}

void MTCP_client::tech_listen(boost::function<bool (SOCKET,void*)> f)
{
    bool exitwh=0;
    while (exitwh==0)
    {
        exitwh=f(sock,datai);
    }
    log.add_log("tech_listen 0");
    reload_sock();
    thread_start=0;
}

void MTCP_client::reload_sock()
{
    closesocket(sock);
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"reload_sock "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("reload_sock 0\n");
    }
}

std::vector<std::string> MTCP_client::print_log()
{
    return log.print_log();
}

void MTCP_client::start_param()
{
    port=0;
    thread_start=0;
}

void MTCP_client::join_in_main()
{
    if (thread_start==1)
        list_thread.join();
}

MTCP_client::~MTCP_client()
{
    if (thread_start==1)
        list_thread.join();
    closesocket(sock);
    Glob.WSA_exit();
}

MUDP_client::MUDP_client()
{
    thread_start=0;
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MUDP_client "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("MUDP_client 0\n");
    }
}

MUDP_client::MUDP_client(std::vector<u_short>porti,std::vector<std::string>ip)
{
    thread_start=0;
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MUDP_client "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        port=porti;
        servip=ip;
        log.add_log("MUDP_client 0\n");
    }
}

void MUDP_client::set_addreses(std::vector<u_short>porti, std::vector<std::string>ip)
{
    port=porti;
    servip=ip;
}

void MUDP_client::add_address(u_short porti,std::string ip)
{
    port.push_back(porti);
    servip.push_back(ip);
}

void MUDP_client::reload_sock()
{
    closesocket(sock);
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"reload_sock "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("reload_sock 0\n");
    }
}

std::vector<std::string> MUDP_client::print_log()
{
    return log.print_log();
}

void MUDP_client::interact_servs(boost::function<bool (SOCKET,std::vector<sockaddr_in>,void*)> f, void* data)
{
    bool err=0;
    datai=data;
    for (u_int i=0; i<servip.size(); i++)
    {
        sockaddr_in serv_addr;
        serv_addr.sin_family=AF_INET;
        serv_addr.sin_port=htons(port[i]);
        if (inet_addr(servip[i].c_str())!=INADDR_NONE)
        {
            serv_addr.sin_addr.s_addr=inet_addr(servip[i].c_str());
            serv_addrs.push_back(serv_addr);
        }
        else
        {
            err=1;
            break;
        }
    }
    if (err==0)
    {
        log.add_log("interact_servs 0\n");
        list_thread=boost::thread ( boost::bind( &MUDP_client::tech_listen, this, f ) );
        thread_start=1;
    }
    else
    {
        std::stringstream sstr;
        sstr<<"interact_servs_ip "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
}

void MUDP_client::tech_listen(boost::function<bool (SOCKET,std::vector<sockaddr_in>,void*)> f)
{
    bool exitwh=0;
    while (exitwh==0)
    {
        exitwh=f(sock,serv_addrs,datai);
    }
    log.add_log("tech_listen 0");
    reload_sock();
    thread_start=0;
}

void MUDP_client::join_in_main()
{
    if (thread_start==1)
        list_thread.join();
}

MUDP_client::~MUDP_client()
{
    if (thread_start==1)
        list_thread.join();
    closesocket(sock);
    Glob.WSA_exit();
}

MTCP_server::MTCP_server()
{
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MTCP_server "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("MTCP_server 0\n");
    }
}

MTCP_server::MTCP_server(u_short porti, std::string ipi)
{
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MTCP_server "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        ip=ipi;
        port=porti;
        addr.sin_family=AF_INET;
        addr.sin_port=htons(port);
        addr.sin_addr.s_addr=inet_addr(ip.c_str());
        log.add_log("MTCP_server 0\n");
    }
}

void MTCP_server::set_address(u_short porti,std::string ipi)
{
    ip=ipi;
    port=porti;
    addr.sin_family=AF_INET;
    addr.sin_port=htons(port);
    addr.sin_addr.s_addr=inet_addr(ip.c_str());
}

void MTCP_server::reload_sock()
{
    closesocket(sock);
    sock=socket(AF_INET,SOCK_STREAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"reload_sock "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
        log.add_log("reload_sock 0\n");
}

void MTCP_server::bindserv()
{
    if (bind(sock,(sockaddr *) &addr,sizeof(addr)))
    {
        std::stringstream sstr;
        sstr<<"bindserv "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
    else
        log.add_log("bindserv 0\n");
}

void MTCP_server::listenserv (boost::function<bool (SOCKET,sockaddr_in,void*)> f,int qri, void* data)
{
    datai=data;
    if (listen(sock, qri))
    {
        std::stringstream sstr;
        sstr<<"listenserv "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
    else
    {
        main_thread=boost::thread ( boost::bind( &MTCP_server::tech_listen, this, f) );
        log.add_log("listenserv 0\n");
    }
}

void MTCP_server::tech_listen(boost::function<bool (SOCKET,sockaddr_in,void*)> f)
{
    SOCKET client_sock;
    sockaddr_in client_addr;
    int client_addr_size=sizeof(client_addr);
    while((client_sock=accept(sock, (sockaddr *)&client_addr, &client_addr_size))!=SOCKET_ERROR)
    {
        boost::thread cl_thread=boost::thread ( boost::bind( &MTCP_server::tech_listen_client, this, f,client_addr,client_sock));
    }
    log.add_log("tech_listen 0 \n");
    reload_sock();
}

void MTCP_server::tech_listen_client(boost::function<bool (SOCKET,sockaddr_in,void*)> f,sockaddr_in client_addr, SOCKET client_sock)
{
    bool exitwh=0;
    while (exitwh==0)
    {
        exitwh=f(client_sock,client_addr,datai);
    }
    closesocket(client_sock);
    log.add_log("tech_listen_client 0");
}

std::vector<std::string> MTCP_server::print_log()
{
    return log.print_log();
}

void MTCP_server::interrupt()
{
    main_thread.interrupt();
}

void MTCP_server::stoplisten()
{
    main_thread.interrupt();
    reload_sock();
}

MTCP_server::~MTCP_server ()
{
    stoplisten();
    closesocket(sock);
    Glob.WSA_exit();
}

MUDP_server::MUDP_server()
{
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MUDP_server "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        log.add_log("MUDP_server 0\n");
    }
}

MUDP_server::MUDP_server(u_short porti, std::string ipi)
{
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"MUDP_server "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
    {
        ip=ipi;
        port=porti;
        addr.sin_family=AF_INET;
        addr.sin_port=htons(port);
        addr.sin_addr.s_addr=inet_addr(ip.c_str());
        log.add_log("MUDP_server 0\n");
    }
}

void MUDP_server::set_address(u_short porti,std::string ipi)
{
    ip=ipi;
    port=porti;
    addr.sin_family=AF_INET;
    addr.sin_port=htons(port);
    addr.sin_addr.s_addr=inet_addr(ip.c_str());
}

void MUDP_server::reload_sock()
{
    closesocket(sock);
    sock=socket(AF_INET,SOCK_DGRAM,0);
    if (sock<0)
    {
        std::stringstream sstr;
        sstr<<"reload_sock "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
    }
    else
        log.add_log("reload_sock 0\n");
}

void MUDP_server::bindserv()
{
    if (bind(sock,(sockaddr *) &addr,sizeof(addr)))
    {
        std::stringstream sstr;
        sstr<<"bindserv "<<WSAGetLastError()<<std::endl;
        log.add_log(sstr.str());
        reload_sock();
    }
    else
        log.add_log("bindserv 0\n");
}

void MUDP_server::interact_clients(boost::function<bool (SOCKET,void*)> f, void* data)
{
    datai=data;
    main_thread=boost::thread ( boost::bind( &MUDP_server::tech_listen, this, f ) );
}

void MUDP_server::tech_listen(boost::function<bool (SOCKET,void*)> f)
{
    bool exitwh=0;
    while (exitwh==0)
    {
        exitwh=f(sock,datai);
    }
    reload_sock();
    log.add_log("tech_listen 0");
}

void MUDP_server::join_in_main()
{
    main_thread.join();
}

void MUDP_server::interrupt()
{
    main_thread.interrupt();
}

MUDP_server::~MUDP_server()
{
    interrupt();
    closesocket(sock);
    Glob.WSA_exit();
#endif
#endif
